/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Florian Zinggeler
 *   Co-authors:
 *      ...
 *
 */

/**
    @file OrxyRoadShip.h
    @brief Declaration of the OrxyRoadShip class.
*/

#ifndef _OrxyRoadShip_H__
#define _OrxyRoadShip_H__


#include "OrxyRoadPrereqs.h"

#include "core/XMLPort.h"
#include "worldentities/pawns/SpaceShip.h"
#include "graphics/Camera.h"

#include "OrxyRoad.h" // Is necessary for getGame function
#include "OrxyRoadCenterPoint.h"

namespace orxonox
{
    class _OrxyRoadExport OrxyRoadShip : public SpaceShip
    {
        public:
            OrxyRoadShip(Context* context);

            virtual void tick(float dt) override;

            // overwrite for 2d movement
            virtual void moveFrontBack(const Vector2& value) override;
            virtual void moveRightLeft(const Vector2& value) override;

            // Starts or stops fireing
            virtual void boost(bool bBoost) override;

            //no rotation!
            virtual void rotateYaw(const Vector2& value) override{};
            virtual void rotatePitch(const Vector2& value) override{};

            //return to main menu if game has ended.
            virtual void rotateRoll(const Vector2& value) override{if (getGame()) if (getGame()->bEndGame) getGame()->end();};

            virtual void updateLevel();

            virtual inline bool collidesAgainst(WorldEntity* otherObject, const btCollisionShape* ownCollisionShape, btManifoldPoint& contactPoint) override;

            float speed, damping, posforeward;
            bool isFireing;
            bool steeredLeft, steeredRight, forward, backward;

        protected:
            virtual void death() override;

        private:
            OrxyRoad* getGame();
            WeakPtr<OrxyRoad> game;
            WeakPtr<WorldEntity> lastEntity;
            Camera* camera;
            float lastTimeFront, lastTimeLeft, lastTime;
            struct Velocity
            {
                float x;
                float y;
            } velocity, desiredVelocity;
           

    };
}

#endif /* _OrxyRoadShip_H__ */
